//
//  keychainAppDelegate.m
//  keychain
//
//  Created by kunii on 11/02/26.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "keychainAppDelegate.h"
#import "RootViewController.h"


@implementation keychainAppDelegate

@synthesize window;
@synthesize navigationController;

/**
*	SecItemCopyMatchingを使った検索
*	@param	[in]	queryDic	検索用辞書
*	@retrun	なし
*/
-(void)seek:(NSMutableDictionary*)queryDic {
	//	検索条件設定
	//	一般的なパスワード
	[queryDic setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];
	//	条件に合う項目すべてをリストアップ
	[queryDic setObject:(id)kSecMatchLimitAll forKey:(id)kSecMatchLimit];
	//	リストする項目の属性情報を取り出す。
	[queryDic setObject:(id)kCFBooleanTrue forKey:(id)kSecReturnAttributes];
	
	//	検索開始！
	NSArray* array = nil;
	OSStatus keychainErr = SecItemCopyMatching((CFDictionaryRef)queryDic, 
		(CFTypeRef *)&array);
	//	結果発表
	if (keychainErr == noErr) {
		for (NSMutableDictionary *dic in array) {
			printf("dic = %s\n", [[dic description] UTF8String]);
			// 返されたNSDataを文字列に変換
			NSData* passwordData = [dic objectForKey:(id)kSecValueData];
			if (passwordData) {
				NSString *password = [[NSString alloc] 
					initWithBytes:[passwordData bytes]
					length:[passwordData length] 
					encoding:NSUTF8StringEncoding];
				printf("password = %s\n", [password UTF8String]);
				[password release];
			}
		}
	} else {
		printf("SecItemCopyMatching error = %d\n", (int)keychainErr);
	}
}

/**
 *	KeyChainサービスの実験
 *	@retrun	なし
 */
-(void)keychainTest {
	//	同じアカウントをkSecClass別に登録できるか？
	NSMutableDictionary* item = [NSMutableDictionary dictionary];
	[item setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];
	[item setObject:@"xcc" forKey:(id)kSecAttrAccount];
	OSStatus status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("kSecClassGenericPassword SecItemAdd error = %d\n", (int)status);

	[item setObject:(id)kSecClassInternetPassword forKey:(id)kSecClass];
	status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("kSecClassInternetPassword SecItemAdd error = %d\n", (int)status);

	//	kSecAttrAccount、kSecAttrServiceの組み合わせ別に登録できるか？
	[item setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];	//	kSecAttrServiceを使うのでkSecClassGenericPasswordに戻す
	[item setObject:@"konohana" forKey:(id)kSecAttrService];
	status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("konohana & xcc SecItemAdd error = %d\n", (int)status);
	
	[item setObject:@"reborn" forKey:(id)kSecAttrAccount];
	status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("konohana & reborn SecItemAdd error = %d\n", (int)status);

	//	konohana & rebornをkSecAttrAccessibleで登録（省略時はkSecAttrAccessibleWhenUnlockedが使われる）
	[item setObject:(id)kSecAttrAccessibleAlways forKey:(id)kSecAttrAccessible];
	status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("konohana & reborn,kSecAttrAccessibleAlways SecItemAdd error = %d\n", (int)status);
	
	//	シミュレータだとerrSecNoAccessForItem:–25243 が返る
	[item setObject:@"ABCDEFG.jp.xcc.konohanaGroup" forKey:(id)kSecAttrAccessGroup];
	status = SecItemAdd((CFDictionaryRef)item, nil);
	printf("konohana & reborn,group SecItemAdd error = %d\n", (int)status);
	
	//	kSecAttrAccount=xcc kSecAttrService="" のコメントを追加
	NSMutableDictionary* updateDic = [NSMutableDictionary dictionary];
	[updateDic setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];
	[updateDic setObject:@"xcc" forKey:(id)kSecAttrAccount];
	NSMutableDictionary* newDic = [NSMutableDictionary dictionary];
	[newDic setObject:(id)@"test comment" forKey:(id)kSecAttrComment];
	status = SecItemUpdate((CFDictionaryRef)updateDic, (CFDictionaryRef)newDic);
	printf("SecItemUpdate error = %d\n", (int)status);
	
	//	検索
	NSMutableDictionary* queryDic = [NSMutableDictionary dictionary];	//	問い合わせ用辞書
	[queryDic setObject:@"konohana" forKey:(id)kSecAttrService];
	printf("list up kSecAttrService=konohana\n");
	[self seek:queryDic];
	[queryDic removeObjectForKey:(id)kSecAttrService];
	[queryDic setObject:@"xcc" forKey:(id)kSecAttrAccount];
	printf("list up kSecAttrAccount = xcc\n");
	[self seek:queryDic];

	//	パスワードの設定
	//	データ：パスワードのNSDataへの変換
	NSString *passwordString = @"test-password";
	[newDic setObject:[passwordString dataUsingEncoding:NSUTF8StringEncoding] forKey:(id)kSecValueData];
	status = SecItemUpdate((CFDictionaryRef)updateDic, (CFDictionaryRef)newDic);
	printf("SecItemUpdate error = %d\n", (int)status);

	//	検索
    [queryDic setObject:(id)kCFBooleanTrue forKey:(id)kSecReturnData];
	[self seek:queryDic];
	
	//	kSecClassGenericPasswordのkSecAttrService:konohana削除
	NSMutableDictionary* deleteDic = [NSMutableDictionary dictionary];
	[deleteDic setObject:(id)kSecClassGenericPassword forKey:(id)kSecClass];
	status = SecItemDelete((CFDictionaryRef)deleteDic);
	printf("kSecClassGenericPassword SecItemDelete error = %d\n", (int)status);
	
	//	kSecClassInternetPasswordのkSecAttrAccount:xcc削除
	[deleteDic setObject:(id)kSecClassInternetPassword forKey:(id)kSecClass];
	[deleteDic removeObjectForKey:(id)kSecAttrService];
	status = SecItemDelete((CFDictionaryRef)deleteDic);
	printf("kSecClassInternetPassword SecItemDelete error = %d\n", (int)status);
}


#pragma mark -
#pragma mark Application lifecycle

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {    
    
    // Override point for customization after application launch.
	[self keychainTest];
    
    // Add the navigation controller's view to the window and display.
    [self.window addSubview:navigationController.view];
    [self.window makeKeyAndVisible];

    return YES;
}


- (void)applicationWillResignActive:(UIApplication *)application {
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}


- (void)applicationDidEnterBackground:(UIApplication *)application {
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     If your application supports background execution, called instead of applicationWillTerminate: when the user quits.
     */
}


- (void)applicationWillEnterForeground:(UIApplication *)application {
    /*
     Called as part of  transition from the background to the inactive state: here you can undo many of the changes made on entering the background.
     */
}


- (void)applicationDidBecomeActive:(UIApplication *)application {
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}


- (void)applicationWillTerminate:(UIApplication *)application {
    /*
     Called when the application is about to terminate.
     See also applicationDidEnterBackground:.
     */
}


#pragma mark -
#pragma mark Memory management

- (void)applicationDidReceiveMemoryWarning:(UIApplication *)application {
    /*
     Free up as much memory as possible by purging cached data objects that can be recreated (or reloaded from disk) later.
     */
}


- (void)dealloc {
	[navigationController release];
	[window release];
	[super dealloc];
}


@end

